document.getElementById("closePopup").addEventListener("click", function(e) {
    e.preventDefault();
    document.body.style.display = "none";
});

document.addEventListener('DOMContentLoaded', function () {
    document.getElementById('searchButton').addEventListener('click', function () {
        console.log("searchButton clicked");

        let tarValue = document.getElementById('tarValue').value.toLowerCase();

        // Check if Tar starts with a dot and remove the dot if true
        if (tarValue.startsWith('.')) {
            tarValue = tarValue.slice(1);
        }

        // Check if Tar starts with a # and remove the # if true
        if (tarValue.startsWith('#')) {
            tarValue = tarValue.slice(1);
        }

        chrome.tabs.query({ active: true, currentWindow: true }, ([tab]) => {
            chrome.scripting.executeScript({
                target: { tabId: tab.id },
                function: runSearchOnPage,
                args: [tarValue]
            }).then((selection) => {
                if (selection && selection.length > 0) {
                    const results = selection[0].result;
                    if (results) {
                        displayResultsInTable("exactResults", results.exact);
                        displayResultsInTable("looseResults", results.loose);

                        // Update the titles if the elements exist
                        const exactMaxElement = document.getElementById('exactMax');
                        const exactAvailableElement = document.getElementById('exactAvailable');
                        const looseMaxElement = document.getElementById('looseMax');
                        const looseAvailableElement = document.getElementById('looseAvailable');

                        if (exactMaxElement) {
                            exactMaxElement.textContent = results.exact.length > 3 ? 3 : results.exact.length;
                        }
                        if (exactAvailableElement) {
                            exactAvailableElement.textContent = results.exact.length;
                        }
                        if (looseMaxElement) {
                            looseMaxElement.textContent = results.loose.length > 3 ? 3 : results.loose.length;
                        }
                        if (looseAvailableElement) {
                            looseAvailableElement.textContent = results.loose.length;
                        }

                        // Display the "show more" link if there are more than MAX_RESULTS results
                        if (results.exact.length > 3) {
                            const showMoreExactElement = document.getElementById('showMoreExact');
                            if (showMoreExactElement) {
                                showMoreExactElement.style.display = 'block';
                                showMoreExactElement.addEventListener('click', function (event) {
                                    event.preventDefault();
                                    displayMoreResults("exactResults", results.exact);
                                });
                            }
                        }
                        if (results.loose.length > 3) {
                            const showMoreLooseElement = document.getElementById('showMoreLoose');
                            if (showMoreLooseElement) {
                                showMoreLooseElement.style.display = 'block';
                                showMoreLooseElement.addEventListener('click', function (event) {
                                    event.preventDefault();
                                    displayMoreResults("looseResults", results.loose);
                                });
                            }
                        }
                    }
                }
            }).catch((error) => {
                console.error(error);
            });
        });
    });

    // Rest of your code remains the same

});

function runSearchOnPage(Tar) {
    const MAX_DISPLAY_COUNT = 5;

    function getUniqueSelectorForElement(element) {
        let selectorParts = [];
        let currentElement = element;

        function nthOfType(element) {
            const siblings = Array.from(element.parentElement.children).filter(child => child.tagName === element.tagName);
            return siblings.indexOf(element) + 1;
        }

        while (currentElement && currentElement.tagName && currentElement !== document.body) {
            let selector = currentElement.tagName.toLowerCase();

            if (document.querySelectorAll(selector).length === 1) {
                selectorParts.unshift(selector);
                let fullSelector = selectorParts.join(' > ');
                if (document.querySelectorAll(fullSelector).length === 1) {
                    return fullSelector;
                }
            } else {
                let attrs = Array.from(currentElement.attributes);
                let optimizedAttrs = [...attrs];
                for (let i = attrs.length - 1; i >= 0; i--) {
                    let tempAttrs = [...optimizedAttrs];
                    tempAttrs.splice(i, 1);
                    let testSelector = selector + tempAttrs.map(attr => `[${attr.name}]`).join('');
                    let fullTestSelector = [...selectorParts, testSelector].join(' > ');
                    if (document.querySelectorAll(fullTestSelector).length === 1) {
                        optimizedAttrs = tempAttrs;
                    }
                }
                let attributeSelector = optimizedAttrs.map(attr => `[${attr.name}]`).join('');
                let combinedSelector = `${selector}${attributeSelector}`;
                if (document.querySelectorAll(combinedSelector).length > 1) {
                    combinedSelector = `${selector}:nth-of-type(${nthOfType(currentElement)})${attributeSelector}`;
                }

                selectorParts.unshift(combinedSelector);
                let fullSelector = selectorParts.join(' > ');

                if (document.querySelectorAll(fullSelector).length === 1) {
                    return fullSelector;
                }
            }
            currentElement = currentElement.parentElement;
        }

        return selectorParts.join(' > ');
    }

    function findMatches(root, tar, exactMatch = true) {
        let results = [];
        function recursiveCheck(element) {
            if (!element || !element.childNodes || !element.attributes) return;

            for (let node of element.childNodes) {
                if (node.nodeType === 3 && node.nodeValue) {
                    let checkCondition = exactMatch ? node.nodeValue.trim().toLowerCase() === tar : node.nodeValue.trim().toLowerCase().includes(tar);
                    if (checkCondition) {
                        let uniqueSelector = getUniqueSelectorForElement(element);
                        if (uniqueSelector) {
                            results.push({ count: 1, attribute: `innerText`, csspath: uniqueSelector });

                        }
                    }
                }
            }

            for (let attr of element.attributes) {
                if (attr && attr.value) {
                    let checkCondition = exactMatch ? attr.value.trim().toLowerCase() === tar : attr.value.trim().toLowerCase().includes(tar);
                    if (checkCondition) {
                        let uniqueSelector = getUniqueSelectorForElement(element);
                        if (uniqueSelector) {
                            results.push({ count: 1, attribute: `${attr.name}`, csspath: uniqueSelector });
                        }
                    }
                }
            }

            if (element.shadowRoot) {
                recursiveCheck(element.shadowRoot);
            }

            for (let child of element.children) {
                recursiveCheck(child);
            }
        }

        recursiveCheck(root);
        return results;
    }



function countOccurrences(root, csspath, attribute) {
    let count = 0;
    let elements = Array.from(root.querySelectorAll(csspath));

    for (let element of elements) {
        if (attribute === 'innerText') {
            if (!element.innerText) {
                console.log("No innerText for element: ", element);
            }
            if (element.innerText && element.innerText.trim().toLowerCase().includes(Tar)) count++;
        } else {
            let attrValue = element.getAttribute(attribute);
            if (attrValue && attrValue.trim().toLowerCase().includes(Tar)) count++;
        }
    }
    return count;
}




    let exactResults = findMatches(document.documentElement, Tar);
    let looseResults = [];

    if (exactResults.length < 10) {
        looseResults = findMatches(document.documentElement, Tar, false);
    }

    looseResults = looseResults.filter(looseResult => {
        return !exactResults.some(exactResult => exactResult.csspath === looseResult.csspath && exactResult.attribute === looseResult.attribute);
    });

    let finalExactResults = [];
    let finalLooseResults = [];

    if (exactResults.length > 0) {
        exactResults.forEach(result => {
            result.count = countOccurrences(document.documentElement, result.csspath, result.attribute);
            finalExactResults.push(result);
        });
        finalExactResults.sort((a, b) => a.csspath.length - b.csspath.length);
    }

    if (looseResults.length > 0) {
        looseResults.forEach(result => {
            result.count = countOccurrences(document.documentElement, result.csspath, result.attribute);
            finalLooseResults.push(result);
        });
        finalLooseResults.sort((a, b) => a.csspath.length - b.csspath.length);
    }

    return {
        exact: finalExactResults,
        loose: finalLooseResults
    };
}

function displayResultsInTable(tableId, results) {
    const tableBody = document.getElementById(tableId);
    const MAX_RESULTS = 3; // maximum number of results to display

    // For updating the titles:
    if (tableId === "exactResults") {
        document.getElementById("exactTitle").textContent = `Exact Match Locators (top ${Math.min(MAX_RESULTS, results.length)} of ${results.length})`;
        if (results.length > MAX_RESULTS) {
            document.getElementById("showMoreExact").style.display = "inline";
        }
    } else if (tableId === "looseResults") {
        document.getElementById("looseTitle").textContent = `Loose Match Locators (top ${Math.min(MAX_RESULTS, results.length)} of ${results.length})`;
        if (results.length > MAX_RESULTS) {
            document.getElementById("showMoreLoose").style.display = "inline";
        }
    }

    // Initial table population code:
    results.slice(0, MAX_RESULTS).forEach(result => {
        const tr = document.createElement('tr');

        const tdCount = document.createElement('td');
        tdCount.textContent = result.count;
        tr.appendChild(tdCount);

        const tdAttribute = document.createElement('td');
        tdAttribute.textContent = result.attribute;
        tr.appendChild(tdAttribute);

        const tdCssPath = document.createElement('td');
        tdCssPath.textContent = result.csspath;
        tr.appendChild(tdCssPath);

        // Dropdown for Action column
        const tdAction = document.createElement('td');
        const dropdown = document.createElement('select');
        dropdown.innerHTML = `
            <option value="select">Select...</option>
            <option value="copy">Copy</option>
            <option value="update">Update Property</option>
            <option value="new">New Property</option>
        `;
        tdAction.appendChild(dropdown);
        tr.appendChild(tdAction);

        tableBody.appendChild(tr);
    });
}

function displayMoreResults(tableId, allResults) {
    const tableBody = document.getElementById(tableId);
    const MAX_RESULTS = 3; // maximum number of results to display

    // Retrieve the current number of displayed rows in the table
    const currentRowCount = tableBody.querySelectorAll('tr').length;

    // Calculate the new maximum limit based on the current row count
    const newMaxLimit = currentRowCount + MAX_RESULTS;

    // Display additional results if available, up to the new maximum limit
    allResults.slice(currentRowCount, newMaxLimit).forEach(result => {
        const tr = document.createElement('tr');

        const tdCount = document.createElement('td');
        tdCount.textContent = result.count;
        tr.appendChild(tdCount);

        const tdAttribute = document.createElement('td');
        tdAttribute.textContent = result.attribute;
        tr.appendChild(tdAttribute);

        const tdCssPath = document.createElement('td');
        tdCssPath.textContent = result.csspath;
        tr.appendChild(tdCssPath);

        // Dropdown for Action column
        const tdAction = document.createElement('td');
        const dropdown = document.createElement('select');
        dropdown.innerHTML = `
            <option value="select">Select...</option>
            <option value="copy">Copy</option>
            <option value="update">Update Property</option>
            <option value="new">New Property</option>
        `;
        tdAction.appendChild(dropdown);
        tr.appendChild(tdAction);

        tableBody.appendChild(tr);
    });

    // If there are no more results to display, hide the "Show More" link
    if (newMaxLimit >= allResults.length) {
        document.getElementById(`showMore${tableId}`).style.display = 'none';
    }

    // Update the titles with the count of displayed results vs. total results
    const displayedCount = currentRowCount + MAX_RESULTS;
    const totalResultsCount = allResults.length;

    if (tableId === 'exactResults') {
        document.getElementById('exactMax').textContent = displayedCount > totalResultsCount ? totalResultsCount : displayedCount;
        document.getElementById('exactAvailable').textContent = totalResultsCount;
    } else if (tableId === 'looseResults') {
        document.getElementById('looseMax').textContent = displayedCount > totalResultsCount ? totalResultsCount : displayedCount;
        document.getElementById('looseAvailable').textContent = totalResultsCount;
    }
}